From a77076e616e175736b4c1ce3e105ae84282a4291 Mon Sep 17 00:00:00 2001
From: aarvid <andy.arvid@gmail.com>
Date: Mon, 4 Feb 2013 14:46:51 -0300
Subject: [PATCH] Fixed bug in gaussian-random when min or max is nil but not
 both.

---
 numbers.lisp |   50 +++++++++++++++++++++++++-------------------------
 1 file changed, 25 insertions(+), 25 deletions(-)

diff --git a/numbers.lisp b/numbers.lisp
index 7210123..30da655 100644
--- a/numbers.lisp
+++ b/numbers.lisp
@@ -18,31 +18,31 @@ normal distribution around 0.0d0.
 Sufficiently positive MIN or negative MAX will cause the algorithm used to
 take a very long time. If MIN is positive it should be close to zero, and
 similarly if MAX is negative it should be close to zero."
-  (labels ((gauss ()
-             (loop
-                for x1 = (- (random 2.0d0) 1.0d0)
-                for x2 = (- (random 2.0d0) 1.0d0)
-                for w = (+ (expt x1 2) (expt x2 2))
-                when (< w 1.0d0)
-                do (let ((v (sqrt (/ (* -2.0d0 (log w)) w))))
-                     (return (values (* x1 v) (* x2 v))))))
-           (guard (x min max)
-             (unless (<= min x max)
-               (tagbody
-                :retry
-                  (multiple-value-bind (x1 x2) (gauss)
-                    (when (<= min x1 max)
-                      (setf x x1)
-                      (go :done))
-                    (when (<= min x2 max)
-                      (setf x x2)
-                      (go :done))
-                    (go :retry))
-                :done))
-             x))
-    (multiple-value-bind (g1 g2) (gauss)
-      (values (guard g1 (or min g1) (or max g1))
-              (guard g2 (or min g2) (or max g2))))))
+  (macrolet ((valid (x) `(<= (or min ,x) ,x (or max ,x)) ))
+    (labels ((gauss ()
+               (loop
+                 for x1 = (- (random 2.0d0) 1.0d0)
+                 for x2 = (- (random 2.0d0) 1.0d0)
+                 for w = (+ (expt x1 2) (expt x2 2))
+                 when (< w 1.0d0)
+                   do (let ((v (sqrt (/ (* -2.0d0 (log w)) w))))
+                        (return (values (* x1 v) (* x2 v))))))
+             (guard (x)
+               (unless (valid x) 
+                 (tagbody
+                  :retry
+                    (multiple-value-bind (x1 x2) (gauss)
+                      (when (valid x1)
+                        (setf x x1)
+                        (go :done))
+                      (when (valid x2)
+                        (setf x x2)
+                        (go :done))
+                      (go :retry))
+                  :done))
+               x))
+      (multiple-value-bind (g1 g2) (gauss)
+        (values (guard g1) (guard g2))))))
 
 (declaim (inline iota))
 (defun iota (n &key (start 0) (step 1))
-- 
1.7.10.4

